import java.io.IOException;
import java.text.DecimalFormat;
import java.util.*;

import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;

/**
 * Classe encarregada de comprovar el nivell de cobertura dels nodes remots mitjançant 
 * el promig dels RSSI (Energy Detection).	
 * @author jterre
 */
public class ControladorCobertura extends TimerTask implements MessageListener {
	
	MoteIF mif;
	int nodeId;
	BatMsg rssi = new BatMsg();
	
	public static final int MAX_CARACTERS_LINIA = 80;   // Longitud de la línia de comandos
	
	/**
	 * Constructor del controlador de cobertura.
	 */
	public ControladorCobertura(){
		PhoenixSource phoenix;
		phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
		
		String source="sf@localhost:9002";	
		phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
		this.mif = new MoteIF(phoenix);
		
		this.mif.registerListener(rssi, this);
	}
	
	public void setNodeId(int nId){
		this.nodeId = nId;
	}
	/**
	 * Lectura del valor ED de cada missatge rebut. La presentació per pantalla es realitza 
	 * mitjançant una línia que s'omple o es buida segons el nivell de cobertura.
	 */
	@Override
	public void messageReceived(int to, Message m) { 

		if(m instanceof BatMsg){
			BatMsg rssi = (BatMsg) m;
			
			if(this.nodeId == rssi.get_nodeid()){
				int ed = rssi.get_rssi();
				int i=1;
				int n=1;
				/* per cobertures màximes */
				if(ed >= MAX_CARACTERS_LINIA){
					while(i<=MAX_CARACTERS_LINIA){
						System.out.print("|");
						i++;
					}
					while(n<=MAX_CARACTERS_LINIA){
						System.out.print("\b");
						n++;
					}
				/* per cobertures mitjanes i mínimes */
				}else{
					while(i<=MAX_CARACTERS_LINIA){
						while(i<=ed){
							System.out.print("|");
							i++;	
						}
						System.out.print("-");
						i++;
					}
					while(n<=MAX_CARACTERS_LINIA){
						System.out.print("\b");
						n++;
					}
				}
			}
		} 
	}
	
	@Override
	public void run() {}
	
	/** Desregistra el Listener al sortir.
	 */
	public void stop(){
		this.mif.deregisterListener(rssi, this);
		System.out.print("\n");
	}
	
}
