
import java.util.*;
import java.io.*;
import net.tinyos.message.Message;
import java.lang.*;

/**
 * Classe amb totes les utilitats i presentacions per pantalla de l'aplicació.
 * @author jterre
 *
 */
public class Interficie  {
	
	/** Valors per defecte **/
	final static int MOSTREIG_TEMP = 5000;
	final static int LLINDAR_TEMP = 0x200;
	final static int MOSTREIG_BAT = 4000; 
	final static int LLINDAR_BAT = 0x01;
	
	final static int MAL_VALOR = 123456789;
	
	ControladorNodes cn;
	int nodeId;
	
	int mostreigTemp = MOSTREIG_TEMP;
	int llindarTemp = LLINDAR_TEMP;
	int mostreigBat = MOSTREIG_BAT;
	int llindarBat = LLINDAR_BAT;
	
	/** Captura els valors que l'usuari entra per teclat **/
	public int lectorValor(){
		BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
		int valor = 0;
		String s = null;
		try{
			s = in.readLine();
			if((s.compareTo("0")>=0 && s.compareTo("9")<=0)){
				valor = Integer.valueOf(s);
			}else{
				valor = MAL_VALOR;
			}
		}catch (IOException e) {
			System.out.println("\n*** Valor incorrecte. ***\n");
		}	
		return valor;  
	  }
	
	/** Instancia el controlador de nodes per inicialitzar-los **/
	public void inicialitzacioNode(int mTemp, int lTemp, int mBat, int lBat){
		this.cn = new ControladorNodes(mostreigTemp, llindarTemp, mostreigBat, llindarTemp);
		cn.run();
	}
	
	/** Guarda al controlador de nodes una nova configuració **/
	public void novaConfiguracio(){
		cn.novaConfiguracio(this.nodeId, this.mostreigTemp, this.llindarTemp, 
					this.mostreigBat, this.llindarBat);	
	}
	
	/** Envia una nova configuració de temperatura **/
	public void configTemp(){
		TransConfigTemp tct;
		int mT, lT;
	
		System.out.println("Configuració sensors temperatura.");
		System.out.print("Introduexi l'identificador del node: ");
		this.nodeId = this.lectorValor();
		System.out.print("Introduexi un valor pel temps de mostreig (segons): ");
		mT = this.lectorValor();
		this.mostreigTemp = mT * 1000;
		System.out.print("Introduexi un valor de llindar (graus ºC): ");
		lT = this.lectorValor();
		this.llindarTemp = this.conversorTempVolts(lT);
		if(this.nodeId == MAL_VALOR || mT == MAL_VALOR || lT == MAL_VALOR){
			System.out.println("\n*** Valors incorrectes. ***\n");
			return;
		}
		tct = new TransConfigTemp(this.nodeId, this.mostreigTemp, this.llindarTemp);
		tct.run();
		this.novaConfiguracio();
	}
	
	/** Envia una nova configuració de bateria **/
	public void configBat(){
		TransConfigBat tcb;
		int mB, lB;
		
		System.out.println("Configuració sensors bateria.");
		System.out.print("Introduexi un valor pel temps de mostreig (segons): ");
		mB = this.lectorValor();
		this.mostreigBat = mB * 1000;
		System.out.print("Introduexi un valor de llindar (volts): ");
		lB = this.lectorValor();
		this.llindarBat = this.conversorBatVolts(lB);
		if(mB == MAL_VALOR || lB == MAL_VALOR){
			System.out.println("\n*** Valors incorrectes. ***\n");
			return;
		}
		this.nodeId = 0;
		tcb = new TransConfigBat(this.nodeId, this.mostreigBat, this.llindarBat);
		tcb.run();
		this.novaConfiguracio();
	}
	
	/** Gestiona un receptor de missatges de temperatura del node n **/
	public void lecturaTemp(ReceptorMissatgesTemp rmt, int nodeId){
		ReceptorMissatgesTemp rm = rmt;
		rm.setNodeId(nodeId);
	}
	
	/** Gestiona un receptor de missatges de bateria del node n **/
	public void lecturaBat(ReceptorMissatgesBat rmb,int nodeId){
		ReceptorMissatgesBat rm = rmb;
		rm.setNodeId(nodeId);
	}
	
	/** Imprimeix per pantalla les configuracions dels nodes **/
	public void printConfiguracions(){
		cn.printConfiguracions();
	}
	
	/** Instancia un receptor dels missatges d'alarma **/
	public void lecturaAlrm(){
		ReceptorMissatgesAlrm rm = new ReceptorMissatgesAlrm();
	}
	
	/** Tractament dels valors de temperatura **/
	public int conversorTempVolts(int temp){	
		return (int)(((temp * 0.01) + 0.5) * (1024 /2.56));
	}
	
	/** Tractament dels valors de bateria **/
	public int conversorBatVolts(int bat){	
		return (int)((bat * 1024)/(2.56 * 2));
	}
	
	/**  Instancia un reconeixedor d'alarmes **/
	public void reconeixerAlrm(int nodeId){
		ReconeixerAlrm ra = new ReconeixerAlrm(this.nodeId);
		ra.run();
	}
	
	/** Gestiona el lector de cobertura **/
	public void lecturaRssi(ControladorCobertura Rssi, int nodeId){
		System.out.println("\nCOBERTURA");
		ControladorCobertura rssi = Rssi;
		rssi.setNodeId(nodeId);
	}
	
	/** Menú d'opcions presentat per pantalla **/
	public void menu(){
		int op;
	    int nodeId;
	    
		do{
			  System.out.println("                             ---- MENU ----                      ");
			  System.out.println("                             ==============                    \n");	  
			  System.out.println("                    1. Configuració sensors temperatura.         ");
			  System.out.println("                    2. Configuració sensors bateria.             ");
			  System.out.println("                    3. Lectura de temperatura.                   ");
			  System.out.println("                    4. Lectura de bateria.                       ");
			  System.out.println("                    5. Llistat configuracions nodes.             ");
			  System.out.println("                    6. Reconeixer alarma.                        ");
			  System.out.println("                    7. Control cobertura.                      \n");
			  
			  System.out.println("                    0. Sortir                                \n\n");
		  

				 System.out.print("\nIntroduexi una opció del menú: ");
				 op = this.lectorValor();
				 System.out.print("\n");
				 
			  switch(op){
			  	case 1: this.configTemp();
			  	break;
			  	
			  	case 2: this.configBat();
			  	break;
			  	
			  	case 3:
			  			ReceptorMissatgesTemp rmt = new ReceptorMissatgesTemp();
			  			System.out.print("Introduexi l'identificador del node: ");
				  		nodeId = this.lectorValor();
				  		System.out.println("Polsi ENTER per tornar a les opcions del menú.\n");
				  		this.lecturaTemp(rmt, nodeId);
				  		this.lectorValor();
				  		rmt.stop();
			  	break;
			  	
			  	case 4: 
			  			ReceptorMissatgesBat rmb = new ReceptorMissatgesBat();
	  					System.out.print("Introduexi l'identificador del node: ");
	  					nodeId = this.lectorValor();
	  					System.out.println("Polsi ENTER per tornar a les opcions del menú.");
	  					this.lecturaBat(rmb, nodeId);
	  					this.lectorValor();
				  		rmb.stop();		
	  			break;
	  			
			  	case 5: this.printConfiguracions();
			  	break;
			  	
			  	case 6: 
			  			System.out.print("Introduexi l'identificador del node: ");
						nodeId = this.lectorValor();
						this.reconeixerAlrm(nodeId);
						System.out.println("Polsi ENTER per tornar a les opcions del menú.");
						this.lectorValor();
				break;
				
			  	case 7: 
			  			ControladorCobertura rssi = new ControladorCobertura();
						System.out.print("Introduexi l'identificador del node: ");
						nodeId = this.lectorValor();
						System.out.println("Polsi ENTER per tornar a les opcions del menú.");
						this.lecturaRssi(rssi, nodeId);
						this.lectorValor();
						rssi.stop();
			  	break;		
			  	
			  	case 0: System.out.println("Sortint de l'aplicació...");	
			  			System.exit(0);
			  	break;
			  	
			  	default: System.out.println("*** Opció invalida. ***\n");
			  }
			 }while(op != 0);		 
	  }


  public static void main(String[] args){
	  
	  	Interficie interficie = new Interficie();
	  	interficie.inicialitzacioNode(interficie.mostreigTemp, interficie.llindarTemp, 
	  									interficie.mostreigBat, interficie.llindarTemp);
	  	
	  	interficie.lecturaAlrm();
	  	
	  	
		  System.out.println("\n******************************************************************************");
		  System.out.println("*                                                                            *");
		  System.out.println("*               SISTEMA DE DETECCIO D'INCENDIS (S.D.I.)                      *");
		  System.out.println("*                                                                            *");
		  System.out.println("******************************************************************************\n");
		  
		interficie.menu();
		  
  }	
}


  