import java.io.IOException;
import java.text.DecimalFormat;
import java.util.*;

import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;

/**
 * Classe encarregada de la recepció de missatges d'alarma.	Retorna un ACK al node emissor al moment
 * en que rep un missatge d'alarma.
 * @author jterre
 *
 */
public class ReceptorMissatgesAlrm extends TimerTask implements MessageListener {
	
	MoteIF mif;
	
	/**
	 * Constructor del receptor.
	 */
	public ReceptorMissatgesAlrm(){
		PhoenixSource phoenix;
		phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
		
		String source="sf@localhost:9002";	
		phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
		this.mif = new MoteIF(phoenix);
		
		this.mif.registerListener(new AlrmMsg(), this);
	}
	
	/** 
	 * Tractament dels valors recollits al missatge.
	 * @param vrefMilivolts
	 * @param counts
	 * @return valor
	 */
	private double passarAVolts(int vrefMilivolts, int counts){
		return (((double)counts)/1024)*(((double)vrefMilivolts)/1000);
	}
	
	/**
	 * Formateig del valor recollit del missatge previament tractat.
	 * @param d
	 * @return valor
	 */
	private String formatejarDouble(double d){
		DecimalFormat df = new DecimalFormat("#0.00");
		return df.format(d);
	}
	
	/** 
	 * Recepció de missatges. Discriminació per tipus i mode d'actuació. Enviament d'ACK al node emissor.
	 */
	@Override
	public void messageReceived(int to, Message m) { 	
		if(m instanceof AlrmMsg){
			AlrmMsg am = (AlrmMsg) m;
			String tipusAlrm = null;
			String modeAct = null;
			String valor = null;

			// envio ACK
			try {
				AckAlrmMsg ack = new AckAlrmMsg();
				ack.set_nodeid(am.get_nodeid());
				mif.send(65535,ack);
			} catch (IOException e) {
				System.err.println("No s'ha pogut enviar el missatge ACK.");
			}
			
			// discriminació per tipus d'alarma
			if(am.get_tipusAlrm() == 1){
				tipusAlrm = "ALRM_TEMP";
				double lecturaVoltsTemp = passarAVolts(2500,am.get_valor());
				double temperaturaCentigrads = (lecturaVoltsTemp-0.5)/0.01;
				valor = formatejarDouble(temperaturaCentigrads) + "ºC";
			}
			if(am.get_tipusAlrm() == 2){
				tipusAlrm = "ALRM_BAT";
				double lecturaVoltsBat = passarAVolts(2500,am.get_valor())*2;
				valor = formatejarDouble(lecturaVoltsBat) + " V";
			}
			
			//discriminació per mode d'actuació
			if(am.get_modeAct() == 3){
				modeAct = "AUTOMATIC";
			}
			if(am.get_modeAct() == 4){
				modeAct = "MANUAL";
			}
			System.out.println("\n\t ALARMA: " + tipusAlrm + " NODE " + am.get_nodeid() + 
									" MODE " + modeAct + " {valor -> " + valor + "}");
			this.printTime();
		}
	}
	
	/** 
	 * Impresió de la data i hora de la recepció de l'alarma.
	 */
	public void printTime(){
		GregorianCalendar gc = new GregorianCalendar();
		int dia = gc.get(Calendar.DAY_OF_MONTH);
		int mes = gc.get(Calendar.MONTH);
		int any = gc.get(Calendar.YEAR);
		int hora = gc.get(Calendar.HOUR_OF_DAY);
		int minut = gc.get(Calendar.MINUTE);
		int segon = gc.get(Calendar.SECOND);
		
		System.out.println("\t\t " + dia + "/" + mes + "/" + any + " " + hora + ":" + minut + ":" + segon);
	}
	
	@Override
	public void run() {}
}
