import java.io.IOException;
import java.text.DecimalFormat;
import java.util.*;

import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;

/**
 * Classe encarregada de la recepció dels missatges de bateria.	
 * @author jterre
 *
 */
public class ReceptorMissatgesBat extends TimerTask implements MessageListener {
	
	MoteIF mif;
	int nodeId;
	BatMsg bm = new BatMsg();
	
	/**
	 * Constructor del receptor.
	 */
	public ReceptorMissatgesBat(){
		PhoenixSource phoenix;
		phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
		
		String source="sf@localhost:9002";	
		phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
		this.mif = new MoteIF(phoenix);
		
		this.mif.registerListener(bm, this);
	}
	
	public void setNodeId(int nId){
		this.nodeId = nId;
	}

	/**
	 * Tractament dels valors recollits al missatge.
	 * @param vrefMilivolts
	 * @param counts
	 * @return valor
	 */
	public double passarAVolts(int vrefMilivolts, int counts){
		return (((double)counts)/1024)*(((double)vrefMilivolts)/1000);
	}
	
	/**
	 * Formateig del valor recollit del missatge previament tractat.
	 * @param d
	 * @return valor
	 */
	public String formatejarDouble(double d){
		DecimalFormat df = new DecimalFormat("#0.00");
		return df.format(d);
	}
	
	/**
	 * Recepció de missatges. Presentació per pantalla del valor de bateria.
	 */
	@Override
	public void messageReceived(int to, Message m) { 

		if(m instanceof BatMsg){
			BatMsg bm = (BatMsg) m;
			
			if(this.nodeId == bm.get_nodeid()){
				double lecturaVoltsBat = passarAVolts(2560,bm.get_bat())*2;				
				System.out.print("Bateria node " + this.nodeId + ": " 
						        + formatejarDouble(lecturaVoltsBat) + " V");
				System.out.print("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
			}
		}
	}
	
	@Override
	public void run() {}
	
	/**
	 *  Desregistra el listener al sortir.
	 */
	public void stop(){
		this.mif.deregisterListener(bm, this);
		System.out.print("\n");
	}
	
}
