import java.text.DecimalFormat;
import java.util.TimerTask;

import net.tinyos.message.Message;
import net.tinyos.message.MessageListener;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;

/**
 * Classe encarregada de la recepció dels missatges de tempertura.	
 * @author jterre
 *
 */
public class ReceptorMissatgesTemp extends TimerTask implements MessageListener {
	
	MoteIF mif;
	int nodeId = -1;
	TempMsg tm = new TempMsg();

	/**
	 * Constructor del receptor.
	 */
	public ReceptorMissatgesTemp(){
		PhoenixSource phoenix;
		phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
		
		String source="sf@localhost:9002";	
		phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
		this.mif = new MoteIF(phoenix);
		
		this.mif.registerListener(tm, this);
	}
	
	public void setNodeId(int nId){
		this.nodeId = nId;
	}
	
	/**
	 * Tractament dels valors recollits al missatge.
	 * @param vrefMilivolts
	 * @param counts
	 * @return valor
	 */
	public double passarAVolts(int vrefMilivolts, int counts){
		return (((double)counts)/1024)*(((double)vrefMilivolts)/1000);
	}
	
	/**
	 * Formateig del valor recollit del missatge previament tractat.
	 * @param d
	 * @return valor
	 */
	public String formatejarDouble(double d){
		DecimalFormat df = new DecimalFormat("#0.00");
		return df.format(d);
	}
	
	/**
	 * Recepció de missatges. Presentació per pantalla del valor de tempertura.
	 */
	@Override
	public void messageReceived(int to, Message m) { 	
		if(m instanceof TempMsg){
			TempMsg tm = (TempMsg) m;
			if(nodeId == tm.get_nodeid()){
				double lecturaVoltsTemp = passarAVolts(2560,tm.get_temp());
				double temperaturaCentigrads = (lecturaVoltsTemp-0.5)/0.01;
				System.out.print("Temperatura node " + tm.get_nodeid() + ": " 
									+formatejarDouble(temperaturaCentigrads) + "ºC");
				System.out.print("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
			}
		}
	}
	
	@Override
	public void run() {}
	
	/**
	 * Desregistra el listener al sortir.
	 */
	public void stop(){
		this.mif.deregisterListener(tm, this);
		System.out.print("\n");
	}
}
